@@ -14,7 +14,6 @@ static const uint8_t KMETER_INTERNAL_TEMP_VAL_REG = 0x10;
 static const uint8_t KMETER_FIRMWARE_VERSION_REG = 0xFE;
 
 void KMeterISOComponent::setup() {
-  ESP_LOGCONFIG(TAG, "Running setup");
   this->error_code_ = NONE;
 
   // Mark as not failed before initializing. Some devices will turn off sensors to save on batteries
@@ -46,7 +45,6 @@ void KMeterISOComponent::setup() {
     this->mark_failed();
     return;
   }
-  ESP_LOGCONFIG(TAG, "The device was successfully setup.");
 }
 
 float KMeterISOComponent::get_setup_priority() const { return setup_priority::DATA; }
