@@ -2,7 +2,7 @@ from esphome.config_validation import Optional
 from esphome.const import CONF_TEXT
 
 from ..defines import CONF_INDICATOR, CONF_MAIN, CONF_PAD_COLUMN
-from ..lv_validation import lv_text, pixels
+from ..lv_validation import lv_text, padding
 from ..lvcode import lv
 from ..schemas import TEXT_SCHEMA
 from ..types import LvBoolean
@@ -19,7 +19,7 @@ class CheckboxType(WidgetType):
             (CONF_MAIN, CONF_INDICATOR),
             TEXT_SCHEMA.extend(
                 {
-                    Optional(CONF_PAD_COLUMN): pixels,
+                    Optional(CONF_PAD_COLUMN): padding,
                 }
             ),
         )
