@@ -31,9 +31,11 @@ void OpenthermNumber::setup() {
 
 void OpenthermNumber::dump_config() {
   LOG_NUMBER("", "OpenTherm Number", this);
-  ESP_LOGCONFIG(TAG, "  Restore value: %d", this->restore_value_);
-  ESP_LOGCONFIG(TAG, "  Initial value: %.2f", this->initial_value_);
-  ESP_LOGCONFIG(TAG, "  Current value: %.2f", this->state);
+  ESP_LOGCONFIG(TAG,
+                "  Restore value: %d\n"
+                "  Initial value: %.2f\n"
+                "  Current value: %.2f",
+                this->restore_value_, this->initial_value_, this->state);
 }
 
 }  // namespace opentherm
