@@ -116,10 +116,7 @@ def wizard_file(**kwargs):
     kwargs["fallback_name"] = ap_name
     kwargs["fallback_psk"] = "".join(random.choice(letters) for _ in range(12))
 
-    if kwargs.get("friendly_name"):
-        base = BASE_CONFIG_FRIENDLY
-    else:
-        base = BASE_CONFIG
+    base = BASE_CONFIG_FRIENDLY if kwargs.get("friendly_name") else BASE_CONFIG
 
     config = base.format(**kwargs)
 
